package javax.swing;

import java.applet.Applet;
import java.awt.Panel;
import java.awt.BorderLayout;
import java.awt.Toolkit;
import java.awt.GraphicsConfiguration;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.List;
import java.awt.Window;
import java.awt.Container;
import sun.awt.EmbeddedFrame;
import java.security.AccessController;
import sun.awt.OSInfo;
import java.awt.GraphicsEnvironment;
import java.awt.Component;

public class PopupFactory
{
    private static final Object SharedInstanceKey;
    private static final int MAX_CACHE_SIZE = 5;
    static final int LIGHT_WEIGHT_POPUP = 0;
    static final int MEDIUM_WEIGHT_POPUP = 1;
    static final int HEAVY_WEIGHT_POPUP = 2;
    private int popupType;
    
    public PopupFactory() {
        this.popupType = 0;
    }
    
    public static void setSharedInstance(PopupFactory popupFactory) {
        if (popupFactory == null) {
            throw new IllegalArgumentException("PopupFactory can not be null");
        }
        SwingUtilities.appContextPut(PopupFactory.SharedInstanceKey, popupFactory);
        return;
    }
    
    public static PopupFactory getSharedInstance() {
        PopupFactory sharedInstance = (PopupFactory)SwingUtilities.appContextGet(PopupFactory.SharedInstanceKey);
        if (sharedInstance == null) {
            sharedInstance = new PopupFactory();
            setSharedInstance(sharedInstance);
            return sharedInstance;
        }
        return sharedInstance;
    }
    
    void setPopupType(int popupType) {
        this.popupType = popupType;
    }
    
    int getPopupType() {
        return this.popupType;
    }
    
    public Popup getPopup(Component component, Component component2, int n, int n2) throws IllegalArgumentException {
        if (component2 == null) {
            throw new IllegalArgumentException("Popup.getPopup must be passed non-null contents");
        }
        Popup popup = this.getPopup(component, component2, n, n2, this.getPopupType(component, component2, n, n2));
        if (popup == null) {
            popup = this.getPopup(component, component2, n, n2, 2);
            return popup;
        }
        return popup;
    }
    
    private int getPopupType(Component parent, Component component, int n, int n2) {
        int popupType = this.getPopupType();
        if (parent == null || this.invokerInHeavyWeightPopup(parent)) {
            popupType = 2;
        }
        else if (popupType == 0 && !(component instanceof JToolTip) && !(component instanceof JPopupMenu)) {
            popupType = 1;
        }
        while (parent != null) {
            if (parent instanceof JComponent && ((JComponent)parent).getClientProperty(ClientPropertyKey.PopupFactory_FORCE_HEAVYWEIGHT_POPUP) == Boolean.TRUE) {
                popupType = 2;
                break;
            }
            parent = parent.getParent();
        }
        return popupType;
    }
    
    private Popup getPopup(Component comp, Component component, int n, int n2, int n3) {
        if (GraphicsEnvironment.isHeadless()) {
            return this.getHeadlessPopup(comp, component, n, n2);
        }
        switch (n3) {
            case 0: {
                return this.getLightWeightPopup(comp, component, n, n2);
            }
            case 1: {
                return this.getMediumWeightPopup(comp, component, n, n2);
            }
            case 2: {
                Popup heavyWeightPopup = this.getHeavyWeightPopup(comp, component, n, n2);
                if (AccessController.doPrivileged(OSInfo.getOSTypeAction()) == OSInfo.OSType.MACOSX && comp != null && EmbeddedFrame.getAppletIfAncestorOf(comp) != null) {
                    ((PopupFactory.HeavyWeightPopup)heavyWeightPopup).setCacheEnabled(false);
                    return heavyWeightPopup;
                }
                return heavyWeightPopup;
            }
            default: {
                return null;
            }
        }
    }
    
    private Popup getHeadlessPopup(Component component, Component component2, int n, int n2) {
        return PopupFactory.HeadlessPopup.getHeadlessPopup(component, component2, n, n2);
    }
    
    private Popup getLightWeightPopup(Component component, Component component2, int n, int n2) {
        return PopupFactory.LightWeightPopup.getLightWeightPopup(component, component2, n, n2);
    }
    
    private Popup getMediumWeightPopup(Component component, Component component2, int n, int n2) {
        return PopupFactory.MediumWeightPopup.getMediumWeightPopup(component, component2, n, n2);
    }
    
    private Popup getHeavyWeightPopup(Component component, Component component2, int n, int n2) {
        if (GraphicsEnvironment.isHeadless()) {
            return this.getMediumWeightPopup(component, component2, n, n2);
        }
        return PopupFactory.HeavyWeightPopup.getHeavyWeightPopup(component, component2, n, n2);
    }
    
    private boolean invokerInHeavyWeightPopup(Component component) {
        if (component != null) {
            Container container = component.getParent();
            while (container != null) {
                if ((container instanceof Popup.HeavyWeightWindow)) {
                    return true;
                }
                container = container.getParent();
            }
        }
        return false;
    }
    
    static {
        SharedInstanceKey = new StringBuffer("PopupFactory.SharedInstanceKey");
    }
    
    private static class HeavyWeightPopup extends Popup
    {
        private static final Object heavyWeightPopupCacheKey;
        private volatile boolean isCacheEnabled;
        
        private HeavyWeightPopup() {
            this.isCacheEnabled = true;
        }
        
        static Popup getHeavyWeightPopup(Component component, Component component2, int n, int n2) {
            Window window = (component != null) ? SwingUtilities.getWindowAncestor(component) : null;
            Object recycledHeavyWeightPopup = null;
            if (window != null) {
                recycledHeavyWeightPopup = getRecycledHeavyWeightPopup(window);
            }
            boolean n3 = false;
            if (component2 != null && component2.isFocusable() && component2 instanceof JPopupMenu) {
                Component[] components = ((JPopupMenu)component2).getComponents();
                int length = components.length;
                int i = 0;
                while (i < length) {
                    Component component3 = components[i];
                    if (!(component3 instanceof MenuElement) && !(component3 instanceof JSeparator)) {
                        n3 = true;
                        break;
                    }
                    ++i;
                }
            }
            if (recycledHeavyWeightPopup == null || (((JWindow)recycledHeavyWeightPopup.getComponent()).getFocusableWindowState() ? 1 : 0) != n3) {
                if (recycledHeavyWeightPopup != null) {
                    recycledHeavyWeightPopup._dispose();
                }
                recycledHeavyWeightPopup = new PopupFactory.HeavyWeightPopup();
            }
            recycledHeavyWeightPopup.reset(component, component2, n, n2);
            if (!(n3 == 0)) {
                JWindow window2 = (JWindow)recycledHeavyWeightPopup.getComponent();
                window2.setFocusableWindowState(true);
                window2.setName("###focusableSwingPopup###");
                return recycledHeavyWeightPopup;
            }
            return recycledHeavyWeightPopup;
        }
        
        private static PopupFactory.HeavyWeightPopup getRecycledHeavyWeightPopup(Window window) {
            synchronized (PopupFactory.HeavyWeightPopup.class) {
                Map<Window, List<PopupFactory.HeavyWeightPopup>> heavyWeightPopupCache = getHeavyWeightPopupCache();
                if (heavyWeightPopupCache.containsKey(window)) {
                    List<Object> list = (List)heavyWeightPopupCache.get(window);
                    if (list.size() > 0) {
                        PopupFactory.HeavyWeightPopup heavyWeightPopup3 = (PopupFactory.HeavyWeightPopup)list.get(0);
                        list.remove(0);
                        return heavyWeightPopup3;
                    }
                    return null;
                }
                else {
                    return null;
                }
            }
        }
        
        private static Map<Window, List<PopupFactory.HeavyWeightPopup>> getHeavyWeightPopupCache() {
            synchronized (PopupFactory.HeavyWeightPopup.class) {
                Map<Object, Object> map = (Map)SwingUtilities.appContextGet(PopupFactory.HeavyWeightPopup.heavyWeightPopupCacheKey);
                if (map == null) {
                    map = new HashMap(2);
                    SwingUtilities.appContextPut(PopupFactory.HeavyWeightPopup.heavyWeightPopupCacheKey, map);
                }
                return map;
            }
        }
        
        private static void recycleHeavyWeightPopup(PopupFactory.HeavyWeightPopup heavyWeightPopup) {
            synchronized (PopupFactory.HeavyWeightPopup.class) {
                Window windowAncestor = SwingUtilities.getWindowAncestor(heavyWeightPopup.getComponent());
                Map<Window, Object> heavyWeightPopupCache = getHeavyWeightPopupCache();
                if (windowAncestor instanceof Popup.DefaultFrame || !windowAncestor.isVisible()) {
                    heavyWeightPopup._dispose();
                    return;
                }
                List<PopupFactory.HeavyWeightPopup> list;
                if (heavyWeightPopupCache.containsKey(windowAncestor)) {
                    list = (List)heavyWeightPopupCache.get(windowAncestor);
                }
                else {
                    list = new ArrayList();
                    heavyWeightPopupCache.put(windowAncestor, list);
                    windowAncestor.addWindowListener(new WindowAdapter() {
                        @Override
                        public void windowClosed(WindowEvent windowEvent) {
                            List<Object> list;
                            synchronized (PopupFactory.HeavyWeightPopup.class) {
                                list = (List)PopupFactory.HeavyWeightPopup.access$000().remove(windowAncestor);
                            }
                            if (list != null) {
                                for (int i = list.size() - 1; i >= 0; --i) {
                                    ((PopupFactory.HeavyWeightPopup)list.get(i))._dispose();
                                }
                            }
                        }
                    });
                }
                if (list.size() < 5) {
                    list.add(heavyWeightPopup);
                }
                else {
                    heavyWeightPopup._dispose();
                }
            }
        }
        
        void setCacheEnabled(boolean isCacheEnabled) {
            this.isCacheEnabled = isCacheEnabled;
        }
        
        @Override
        public void hide() {
            super.hide();
            if (this.isCacheEnabled) {
                recycleHeavyWeightPopup(this);
            }
            else {
                this._dispose();
            }
        }
        
        @Override
        void dispose() {
        }
        
        void _dispose() {
            super.dispose();
        }
        
        static Map access$000() {
            return getHeavyWeightPopupCache();
        }
        
        static {
            heavyWeightPopupCacheKey = new StringBuffer("PopupFactory.heavyWeightPopupCache");
        }
    }
    
    private static class ContainerPopup extends Popup
    {
        Component owner;
        int x;
        int y;
        
        @Override
        public void hide() {
            Component component = this.getComponent();
            if (component != null) {
                Container parent = component.getParent();
                if (parent != null) {
                    Rectangle bounds = component.getBounds();
                    parent.remove(component);
                    parent.repaint(bounds.x, bounds.y, bounds.width, bounds.height);
                }
            }
            this.owner = null;
        }
        
        public void pack() {
            Component component = this.getComponent();
            if (component != null) {
                component.setSize(component.getPreferredSize());
            }
        }
        
        @Override
        void reset(Component layeredPane, Component component, int x, int y) {
            if (layeredPane instanceof JFrame || layeredPane instanceof JDialog || layeredPane instanceof JWindow) {
                layeredPane = ((RootPaneContainer)layeredPane).getLayeredPane();
            }
            super.reset(layeredPane, component, x, y);
            this.x = x;
            this.y = y;
            this.owner = layeredPane;
        }
        
        boolean overlappedByOwnedWindow() {
            Component component = this.getComponent();
            if (this.owner != null && component != null) {
                Window windowAncestor = SwingUtilities.getWindowAncestor(this.owner);
                if (windowAncestor == null) {
                    return false;
                }
                Window[] ownedWindows = windowAncestor.getOwnedWindows();
                if (ownedWindows != null) {
                    Rectangle bounds = component.getBounds();
                    int length = ownedWindows.length;
                    int i = 0;
                    while (i < length) {
                        Window window = ownedWindows[i];
                        if (window.isVisible() && bounds.intersects(window.getBounds())) {
                            return true;
                        }
                        ++i;
                    }
                }
            }
            return false;
        }
        
        boolean fitsOnScreen() {
            boolean n = false;
            Component component = this.getComponent();
            if (this.owner != null && component != null) {
                int width = component.getWidth();
                int height = component.getHeight();
                Container container = (Container)SwingUtilities.getRoot(this.owner);
                if (container instanceof JFrame || container instanceof JDialog || container instanceof JWindow) {
                    Rectangle bounds2 = container.getBounds();
                    Insets insets = container.getInsets();
                    bounds2.x += insets.left;
                    bounds2.y += insets.top;
                    bounds2.width -= insets.left + insets.right;
                    bounds2.height -= insets.top + insets.bottom;
                    if (JPopupMenu.canPopupOverlapTaskBar()) {
                        n = bounds2.intersection(this.getContainerPopupArea(container.getGraphicsConfiguration())).contains(this.x, this.y, width, height);
                    }
                    else {
                        n = bounds2.contains(this.x, this.y, width, height);
                    }
                }
                else if ((container instanceof JApplet)) {
                    Rectangle bounds = container.getBounds();
                    Point locationOnScreen = container.getLocationOnScreen();
                    bounds.x = locationOnScreen.x;
                    bounds.y = locationOnScreen.y;
                    n = bounds.contains(this.x, this.y, width, height);
                }
            }
            return n;
        }
        
        Rectangle getContainerPopupArea(GraphicsConfiguration gc) {
            Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
            Rectangle bounds;
            Insets screenInsets;
            if (gc != null) {
                bounds = gc.getBounds();
                screenInsets = defaultToolkit.getScreenInsets(gc);
            }
            else {
                bounds = new Rectangle(defaultToolkit.getScreenSize());
                screenInsets = new Insets(0, 0, 0, 0);
            }
            bounds.x += screenInsets.left;
            bounds.y += screenInsets.top;
            bounds.width -= screenInsets.left + screenInsets.right;
            bounds.height -= screenInsets.top + screenInsets.bottom;
            return bounds;
        }
    }
    
    private static class HeadlessPopup extends PopupFactory.ContainerPopup
    {
        static Popup getHeadlessPopup(Component component, Component component2, int n, int n2) {
            PopupFactory.HeadlessPopup headlessPopup = new PopupFactory.HeadlessPopup();
            headlessPopup.reset(component, component2, n, n2);
            return headlessPopup;
        }
        
        @Override
        Component createComponent(Component component) {
            return new Panel(new BorderLayout());
        }
        
        @Override
        public void show() {
        }
        
        @Override
        public void hide() {
        }
    }
    
    private static class LightWeightPopup extends PopupFactory.ContainerPopup
    {
        private static final Object lightWeightPopupCacheKey;
        
        static Popup getLightWeightPopup(Component component, Component component2, int n, int n2) {
            PopupFactory.LightWeightPopup recycledLightWeightPopup = getRecycledLightWeightPopup();
            if (recycledLightWeightPopup == null) {
                recycledLightWeightPopup = new PopupFactory.LightWeightPopup();
            }
            recycledLightWeightPopup.reset(component, component2, n, n2);
            if (!recycledLightWeightPopup.fitsOnScreen() || recycledLightWeightPopup.overlappedByOwnedWindow()) {
                recycledLightWeightPopup.hide();
                return null;
            }
            return recycledLightWeightPopup;
        }
        
        private static List<PopupFactory.LightWeightPopup> getLightWeightPopupCache() {
            List<Object> list = (List)SwingUtilities.appContextGet(PopupFactory.LightWeightPopup.lightWeightPopupCacheKey);
            if (list == null) {
                list = new ArrayList();
                SwingUtilities.appContextPut(PopupFactory.LightWeightPopup.lightWeightPopupCacheKey, list);
                return list;
            }
            return list;
        }
        
        private static void recycleLightWeightPopup(PopupFactory.LightWeightPopup lightWeightPopup) {
            synchronized (PopupFactory.LightWeightPopup.class) {
                List<PopupFactory.LightWeightPopup> lightWeightPopupCache = getLightWeightPopupCache();
                if (lightWeightPopupCache.size() < 5) {
                    lightWeightPopupCache.add(lightWeightPopup);
                }
            }
        }
        
        private static PopupFactory.LightWeightPopup getRecycledLightWeightPopup() {
            synchronized (PopupFactory.LightWeightPopup.class) {
                List<PopupFactory.LightWeightPopup> lightWeightPopupCache = getLightWeightPopupCache();
                if (lightWeightPopupCache.size() > 0) {
                    PopupFactory.LightWeightPopup lightWeightPopup2 = (PopupFactory.LightWeightPopup)lightWeightPopupCache.get(0);
                    lightWeightPopupCache.remove(0);
                    return lightWeightPopup2;
                }
                return null;
            }
        }
        
        @Override
        public void hide() {
            super.hide();
            ((Container)this.getComponent()).removeAll();
            recycleLightWeightPopup(this);
        }
        
        @Override
        public void show() {
            Container container = null;
            if (this.owner != null) {
                container = ((this.owner instanceof Container) ? ((Container)this.owner) : this.owner.getParent());
            }
            for (; container != null; container = container.getParent()) {
                if ((container instanceof JRootPane)) {
                    if (!(container.getParent() instanceof JInternalFrame)) {
                        container = ((JRootPane)container).getLayeredPane();
                    }
                }
                else if ((container instanceof Window)) {
                    if (container == null) {
                        break;
                    }
                    break;
                }
                else if ((container instanceof JApplet)) {
                    break;
                }
            }
            Point convertScreenLocationToParent = SwingUtilities.convertScreenLocationToParent(container, this.x, this.y);
            Component component = this.getComponent();
            component.setLocation(convertScreenLocationToParent.x, convertScreenLocationToParent.y);
            if ((container instanceof JLayeredPane)) {
                container.add(component, JLayeredPane.POPUP_LAYER, 0);
            }
            else {
                container.add(component);
            }
        }
        
        @Override
        Component createComponent(Component component) {
            JPanel panel = new JPanel(new BorderLayout(), true);
            panel.setOpaque(true);
            return panel;
        }
        
        @Override
        void reset(Component component, Component comp, int x, int y) {
            super.reset(component, comp, x, y);
            JComponent component2 = (JComponent)this.getComponent();
            component2.setOpaque(comp.isOpaque());
            component2.setLocation(x, y);
            component2.add(comp, "Center");
            comp.invalidate();
            this.pack();
        }
        
        static {
            lightWeightPopupCacheKey = new StringBuffer("PopupFactory.lightPopupCache");
        }
    }
    
    private static class MediumWeightPopup extends PopupFactory.ContainerPopup
    {
        private static final Object mediumWeightPopupCacheKey;
        private JRootPane rootPane;
        
        static Popup getMediumWeightPopup(Component component, Component component2, int n, int n2) {
            PopupFactory.MediumWeightPopup recycledMediumWeightPopup = getRecycledMediumWeightPopup();
            if (recycledMediumWeightPopup == null) {
                recycledMediumWeightPopup = new PopupFactory.MediumWeightPopup();
            }
            recycledMediumWeightPopup.reset(component, component2, n, n2);
            if (!recycledMediumWeightPopup.fitsOnScreen() || recycledMediumWeightPopup.overlappedByOwnedWindow()) {
                recycledMediumWeightPopup.hide();
                return null;
            }
            return recycledMediumWeightPopup;
        }
        
        private static List<PopupFactory.MediumWeightPopup> getMediumWeightPopupCache() {
            List<Object> list = (List)SwingUtilities.appContextGet(PopupFactory.MediumWeightPopup.mediumWeightPopupCacheKey);
            if (list == null) {
                list = new ArrayList();
                SwingUtilities.appContextPut(PopupFactory.MediumWeightPopup.mediumWeightPopupCacheKey, list);
                return list;
            }
            return list;
        }
        
        private static void recycleMediumWeightPopup(PopupFactory.MediumWeightPopup mediumWeightPopup) {
            synchronized (PopupFactory.MediumWeightPopup.class) {
                List<PopupFactory.MediumWeightPopup> mediumWeightPopupCache = getMediumWeightPopupCache();
                if (mediumWeightPopupCache.size() < 5) {
                    mediumWeightPopupCache.add(mediumWeightPopup);
                }
            }
        }
        
        private static PopupFactory.MediumWeightPopup getRecycledMediumWeightPopup() {
            synchronized (PopupFactory.MediumWeightPopup.class) {
                List<PopupFactory.MediumWeightPopup> mediumWeightPopupCache = getMediumWeightPopupCache();
                if (mediumWeightPopupCache.size() > 0) {
                    PopupFactory.MediumWeightPopup mediumWeightPopup2 = (PopupFactory.MediumWeightPopup)mediumWeightPopupCache.get(0);
                    mediumWeightPopupCache.remove(0);
                    return mediumWeightPopup2;
                }
                return null;
            }
        }
        
        @Override
        public void hide() {
            super.hide();
            this.rootPane.getContentPane().removeAll();
            recycleMediumWeightPopup(this);
        }
        
        @Override
        public void show() {
            Component component = this.getComponent();
            Container container = null;
            if (this.owner != null) {
                container = this.owner.getParent();
            }
            while (!(container instanceof Window) && !(container instanceof Applet) && container != null) {
                container = container.getParent();
            }
            if ((container instanceof RootPaneContainer)) {
                JLayeredPane layeredPane = ((RootPaneContainer)container).getLayeredPane();
                Point convertScreenLocationToParent2 = SwingUtilities.convertScreenLocationToParent(layeredPane, this.x, this.y);
                component.setVisible(false);
                component.setLocation(convertScreenLocationToParent2.x, convertScreenLocationToParent2.y);
                layeredPane.add(component, JLayeredPane.POPUP_LAYER, 0);
            }
            else {
                Point convertScreenLocationToParent = SwingUtilities.convertScreenLocationToParent(container, this.x, this.y);
                component.setLocation(convertScreenLocationToParent.x, convertScreenLocationToParent.y);
                component.setVisible(false);
                container.add(component);
            }
            component.setVisible(true);
        }
        
        @Override
        Component createComponent(Component component) {
            PopupFactory.MediumWeightPopup.MediumWeightComponent mediumWeightComponent = new PopupFactory.MediumWeightPopup.MediumWeightComponent();
            (this.rootPane = new JRootPane()).setOpaque(true);
            mediumWeightComponent.add(this.rootPane, "Center");
            return mediumWeightComponent;
        }
        
        @Override
        void reset(Component component, Component comp, int x, int y) {
            super.reset(component, comp, x, y);
            Component component2 = this.getComponent();
            component2.setLocation(x, y);
            this.rootPane.getContentPane().add(comp, "Center");
            comp.invalidate();
            component2.validate();
            this.pack();
        }
        
        static {
            mediumWeightPopupCacheKey = new StringBuffer("PopupFactory.mediumPopupCache");
        }
        
        private static class MediumWeightComponent extends Panel implements SwingHeavyWeight
        {
            MediumWeightComponent() {
                super(new BorderLayout());
            }
        }
    }
}
